/*
 * Copyright(C) OASIS(R) 2005,2009. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 */
package client;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.net.URL;

import javax.xml.namespace.QName;
import javax.xml.ws.Service;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.oasisopen.sca.test.ASM_0001_Client;

import testClient.TestException;
import testClient.TestException_Exception;
import testClient.TestInvocation;

/**
 * A generic test client based on JAX-WS APIs
 */
public class BaseJAXWSTestCase {

	private static RuntimeBridge runtimeBridge = null;
	private static String RUNTIME_BRIDGE_IMPL_PROPKEY = "OASIS_TESTENV_RUNTIME_BRIDGE_CLASS";
	private static String RUNTIME_IMPL_LANG_PROPKEY = "OASIS_TESTENV_IMPL_LANG";
	protected static String _Lang = "_Java";

	// Load the Bridge class.
	static {
		setupImplLanguage();
		try {
			setupRuntimeBridge();
		} catch (Exception e) {
			System.err.println("Failed to create a bridge class to the runtime under test. " +
					           "Please ensure the environment variable or System property named " +
					           "OASIS_TESTENV_RUNTIME_BRIDGE_CLASS is set to a valid implementation of the client.RuntimeBridge interface.");
			e.printStackTrace();
		} // end try
	} // end static
	
	private static void setupImplLanguage() {
		String implLanguage = System.getenv(RUNTIME_IMPL_LANG_PROPKEY);
		if (implLanguage == null) implLanguage = System.getProperty(RUNTIME_IMPL_LANG_PROPKEY);
		if (implLanguage == null) { 
			System.err.println("No implementation language specified - using 'Java'." +
			                   "Set the OASIS_TESTENV_IMPL_LANG environment variable or System property.");
		} else {
			_Lang = implLanguage;
			System.out.println("Implementation language set to: " + _Lang);
			_Lang = "_" + _Lang;
		} // end if			                                    
	} // end setupImplLanguage
	
	/**
	 * Sets up a connection to a Runtime Bridge class which provides access to the SCA runtime, enabling the runtime to be started
	 * and contributions to be passed to the runtime and started
	 * @return true if the Runtime Bridge was found and loaded, false if the bridge could not be found
	 * Uses a System Variable or an Environment Variable by the name "OASIS_TESTENV_RUNTIME_BRIDGE_CLASS" to name the Bridge class
	 */
	private static boolean setupRuntimeBridge() throws Exception {

		String runtimeBridgeClass = System.getenv(RUNTIME_BRIDGE_IMPL_PROPKEY);
		if (runtimeBridgeClass == null) runtimeBridgeClass = System.getProperty(RUNTIME_BRIDGE_IMPL_PROPKEY);
		if( runtimeBridgeClass == null ) {
			// Print message and select the Tuscany Bridge as a default
			System.err.println("Set System Variable or Environment Variable 'OASIS_TESTENV_RUNTIME_BRIDGE_CLASS' to name of runtime bridge class "); 
			runtimeBridgeClass = "client.TuscanyRuntimeBridge";
		} // end if 
		
		if (runtimeBridgeClass != null) {
			try {
				runtimeBridge = (RuntimeBridge) Class.forName(
						runtimeBridgeClass).newInstance();
				return true;
			} catch (ClassNotFoundException e) {
				System.err.println("Runtime bridge class \"" + runtimeBridgeClass + "\" not found");
			}
		} // end if
		return false;
	} // end method setupRuntimeBridge
	
    protected TestConfiguration testConfiguration = getTestConfiguration();
    static boolean proceed = true;
    
    public static void main(String[] args) throws Exception {
    	BaseJAXWSTestCase test = new BaseJAXWSTestCase(); 
    	test.setUp();
    	test.tearDown();
    } // end method main
    
    /**
     * Set up the runtime - pass the configuration to the runtime bridge and then start the contribution(s) which represent
     * the SCA application
     * @throws Exception
     */
    @Before
    public void setUp() throws Exception {
    	runtimeBridge.setTestConfiguration(getTestConfiguration());
    	// Don't proceed to run the test unless the contribution loaded successfully... 
    	proceed = false;
    	try {
    		if ( startContribution() ) proceed = true; 
    	} catch (Exception e) {
    		// If the SCA runtime refuses to start an invalid contribution, then this is also
    		// regarded as a successful outcome, if the expected outcome is marked as "exception"
   			System.out.println( "Exception received during startup - detail: " + e.getMessage() );
    		assertEquals( testConfiguration.getExpectedOutput()[0], "exception" );
    		System.out.println("Test " + testConfiguration.getTestName() + " completed successfully");
    	} // end try
    }

    @After
    public void tearDown() throws Exception {
    	stopContribution();
    }

    /**
     * Run the test, if the initialization of the SCA application was successful
     * @throws Exception - if the output from the application does not match the expected output
     */
    @Test
    public void testDummy() throws Exception {
    	// If an exception were thrown during initialization, let's go no further
    	if( proceed == false ) return; //TODO - check, should we be failing here?
    	
    	// System.out.println("Test " + testName + " starting");
    	String output = null;
    	try {
	    	output = invokeTest( testConfiguration.getInput() );

    	} catch ( TestException_Exception e ) {
    		TestException exceptionContent = e.getFaultInfo();
    		System.out.println("Service fault received - detail: " + exceptionContent.getMessage() );
    		assertEquals( testConfiguration.getExpectedOutput()[0], "exception" );
    		System.out.println("Test " + testConfiguration.getTestName() + " completed successfully");
    		return;
    	} catch (Throwable e) {
    		e.printStackTrace();
   			System.out.println( "Exception received - detail: " + e.getMessage() );
    		assertEquals( testConfiguration.getExpectedOutput()[0], "exception" );
    		System.out.println("Test " + testConfiguration.getTestName() + " completed successfully");
    		return;
    	}
    	// If there are multiple outputs possible, check each one until there is a match
    	String[] expectedOutput = testConfiguration.getExpectedOutput();
    	AssertionError e = null;
    	
		for( String expected : expectedOutput ) {
			e = null;
    		try {
    			assertEquals( expected, output );
    		} catch ( AssertionError eReturned ) {
    			e = eReturned;
    		} //end try
    		if ( e == null ) break;
		} // end for 
		if( e != null ) throw e;
    	System.out.println("Test " + testConfiguration.getTestName() + " completed successfully");
    }
    
    /**
     * Invoke the test via a JAX-WS web services interface
     * @param input - input string
     * @return the output string from the web service invocation
     * @throws Exception - if the Web service invocation fails (e.g. the target service does not exist)
     */
    public String invokeTest( String input ) throws Exception {
    	//Web service invocation via JAXWS
    	QName serviceName = new QName("http://test.sca.oasisopen.org/", "TestInvocationService");
    	URL wsdlLocation = this.getClass().getClassLoader().getResource("TestClient.wsdl");
    	javax.xml.ws.Service webService = Service.create( wsdlLocation, serviceName );
    	TestInvocation wsProxy = (TestInvocation) webService.getPort(testConfiguration.getServiceInterface());

    	String output = wsProxy.invokeTest(input);

    	return output;
    } // end method invokeTest
    

    /**
     * Start the supplied contribution(s) via the Runtime Bridge
     * @return true if the contributions were started sucessfully
     * @throws Exception
     */
    protected boolean startContribution() throws Exception {
    	
    	return runtimeBridge.startContribution(getContributionURIs());
    	
    } // end method startContribution
    
    /**
     * Stop the SCA application
     * @throws Exception if there was a problem stopping the original application
     */
    protected void stopContribution() throws Exception {
    		runtimeBridge.stopContribution();
    } // end method stopContribution
    
    /**
     * Gets the location of the Contributions as File Paths
     * @return the contribution locations
     */
    protected String[] getContributionURIs() {
    	String[] locations;
    	locations = testConfiguration.getContributionNames();
    	
    	if( locations != null ) {
    		// If specified, the Contribution names in the test configuration are relative to the
    		// base directory of the testcases
    		URL codeURL = this.getClass().getProtectionDomain().getCodeSource().getLocation();
	    	String dirFile = codeURL.getFile();
	    	
	    	File thisFile = new File( dirFile );
	    	thisFile = thisFile.getParentFile();
	    	
    		for( int i=0; i < locations.length; i++ ) {
		    	File contributionFile = new File( thisFile, locations[i] );
		    	
		    	locations[i] = contributionFile.toURI().toString();
    		} // end for    	  	
    	} else {
    		// No contribution specified - use the default one
        	locations = new String[1];
        	locations[0] = runtimeBridge.getContributionLocation(testConfiguration.getTestClass());
        	testConfiguration.contributionNames = new String[] { "default" };
    	} // end if
	    	
 
    	return locations;
    } // end getContributionURI
    
    /**
     * The test configuration - this method is subclassed by individual testcases to provide detail about the test to run
     * @return a Test Configuration data structure
     */
    protected TestConfiguration getTestConfiguration() {
    	TestConfiguration config = new TestConfiguration();
    	// The name of the test
    	config.testName 		= "ASM_0001";
    	// The input string for the test
    	config.input 			= "request";
    	// The expected output string from the test
    	config.output[0] 			= config.testName + " " + config.input + " invoked ok";
    	// The name of the composite to run for the SCA application under test
    	config.composite 		= "Test_ASM_0101.composite";
    	// The test service to invoke on the SCA application side
    	config.testServiceName 	= "TestClient";
    	//
    	config.testClass 		= ASM_0001_Client.class;
    	// An array of contribution names
    	config.contributionNames = null;
    	// The client service invocation class
    	config.serviceInterface = TestInvocation.class;
    	return config;
    }
    
} // end class BaseTest
